@@ -15,15 +15,14 @@ linking mixed C++/Fortran use either Fortran (Intel) or
 C++ (GNU) compiler.
 """
 
-from __future__ import (
-    print_function, division, absolute_import, unicode_literals
-)
+from __future__ import print_function, division, absolute_import
 
 import glob
 import os
 import shutil
 import sys
 import tempfile
+import warnings
 
 from .util import (
     MetaReaderWriter, missing_or_other_newer, get_abspath,
@@ -39,14 +38,19 @@ from .runners import (
     FortranCompilerRunner
 )
 
+from distutils.sysconfig import get_config_var
+
+# if sys.version_info[0] == 2:  # python 2
+sharedext = get_config_var('SO')
+# else:
+#    sharedext = get_config_var('EXT_SUFFIX')
+
 if os.name == 'posix':  # Future improvement to make cross-platform
-    flagprefix = '-'
+    # flagprefix = '-'
     objext = '.o'
-    sharedext = '.so'
 elif os.name == 'nt':
     # flagprefix = '/' <-- let's assume mingw compilers...
     objext = '.obj'
-    sharedext = '.dll'
 else:
     raise ImportError("Unknown os.name: {}".format(os.name))
 
@@ -90,7 +94,7 @@ def compile_sources(files, CompilerRunner_=None,
     Compile source code files to object files.
 
     Parameters
-    ==========
+    ----------
     files: iterable of path strings
         source files, if cwd is given, the paths are taken as relative.
     CompilerRunner_: CompilerRunner instance (optional)
@@ -155,7 +159,7 @@ def link(obj_files, out_file=None, shared=False, CompilerRunner_=None,
     Link object files.
 
     Parameters
-    ==========
+    ----------
     obj_files: iterable of path strings
     out_file: path string (optional)
         path to executable/shared library, if missing
@@ -175,7 +179,7 @@ def link(obj_files, out_file=None, shared=False, CompilerRunner_=None,
         keyword arguments passed onto CompilerRunner_
 
     Returns
-    =======
+    -------
     The absolute to the generated shared object / executable
 
     """
@@ -205,6 +209,12 @@ def link(obj_files, out_file=None, shared=False, CompilerRunner_=None,
     if shared:
         if '-shared' not in flags:
             flags.append('-shared')
+
+        # mimic GNU linker behavior on OS X when using -shared
+        # (otherwise likely Undefined symbol errors)
+        dl_flag = '-undefined dynamic_lookup'
+        if sys.platform == 'darwin' and dl_flag not in flags:
+            flags.append(dl_flag)
     run_linker = kwargs.pop('run_linker', True)
     if not run_linker:
         raise ValueError("link(..., run_linker=False)!?")
@@ -224,7 +234,7 @@ def link_py_so(obj_files, so_file=None, cwd=None, libraries=None,
     Link python extension module (shared object) for importing
 
     Parameters
-    ==========
+    ----------
     obj_files: iterable of path strings
         object files to be linked
     so_file: path string
@@ -244,38 +254,53 @@ def link_py_so(obj_files, so_file=None, cwd=None, libraries=None,
         keyword arguments passed onto `link(...)`
 
     Returns
-    =======
+    -------
     Absolute path to the generate shared object
     """
     libraries = libraries or []
 
-    from distutils.sysconfig import get_config_vars
     include_dirs = kwargs.pop('include_dirs', [])
     library_dirs = kwargs.pop('library_dirs', [])
+    # from distutils/command/build_ext.py:
+    if sys.platform == "win32":
+        warnings.warn("Windows not yet supported.")
+    elif sys.platform == 'darwin':
+        # Don't use the default code below
+        pass
+    elif sys.platform[:3] == 'aix':
+        # Don't use the default code below
+        pass
+    elif get_config_var('Py_ENABLE_SHARED'):
+        # LIBDIR/INSTSONAME should always points to libpython (dynamic or static)
+        pylib = os.path.join(get_config_var('LIBDIR'), get_config_var('INSTSONAME'))
+        if os.path.exists(pylib):
+            libraries.append(pylib)
+        else:
+            ABIFLAGS = get_config_var('ABIFLAGS')
+            pythonlib = 'python{}.{}{}'.format(
+                sys.hexversion >> 24, (sys.hexversion >> 16) & 0xff,
+                ABIFLAGS or '')
+            libraries += [pythonlib]
 
-    # Anaconda Win 32 returns a very terse dict from:
-    # get_config_vars(), hence we are (arbitrarily assuming mingw32
-    # in order to at least have a small chance of build succeeding)
-
-    BLDLIBRARY = get_config_vars('BLDLIBRARY')[0] or (
-        '-lpython'+str(sys.version_info.major)+str(sys.version_info.minor))
-    BLDSHARED = get_config_vars('BLDSHARED')[0] or 'gcc -shared'
-    libraries += [x[2:] for x in BLDLIBRARY.split() if x.startswith('-l')]
-    cc = BLDSHARED
+    flags = kwargs.pop('flags', [])
+    needed_flags = ('-pthread',)
+    for flag in needed_flags:
+        if flag not in flags:
+            flags.append(flag)
 
     # We want something like: gcc, ['-pthread', ...
-    compilername, flags = cc.split()[0], cc.split()[1:]
+    # compilername, flags = cc.split()[0], cc.split()[1:]
 
-    # Grab include_dirs
-    include_dirs += list(filter(lambda x: x.startswith('-I'), flags))
-    flags = list(filter(lambda x: not x.startswith('-I'), flags))
+    # # Grab include_dirs
+    # include_dirs += list(filter(lambda x: x.startswith('-I'), flags))
+    # flags = list(filter(lambda x: not x.startswith('-I'), flags))
 
-    # Grab library_dirs
-    library_dirs += [x[2:] for x in filter(
-        lambda x: x.startswith('-L'), flags)]
-    flags = list(filter(lambda x: not x.startswith('-L'), flags))
+    # # Grab library_dirs
+    # library_dirs += [x[2:] for x in filter(
+    #     lambda x: x.startswith('-L'), flags)]
+    # flags = list(filter(lambda x: not x.startswith('-L'), flags))
 
-    flags.extend(kwargs.pop('flags', []))
+    # flags.extend(kwargs.pop('flags', []))
 
     return link(obj_files, shared=True, flags=flags, cwd=cwd,
                 cplus=cplus, fort=fort, include_dirs=include_dirs,
@@ -289,7 +314,7 @@ def simple_cythonize(src, destdir=None, cwd=None, logger=None,
     Generates a C file from a Cython source file.
 
     Parameters
-    ==========
+    ----------
     src: path string
         path to Cython source
     destdir: path string (optional)
@@ -304,7 +329,7 @@ def simple_cythonize(src, destdir=None, cwd=None, logger=None,
         Only cythonize if source is newer. default: False
     **cy_kwargs:
         second argument passed to cy_compile.
-        Generates a .cpp file is cplus=True in cy_kwargs, else a .c file.
+        Generates a .cpp file if cplus=True in cy_kwargs, else a .c file.
     """
     from Cython.Compiler.Main import (
         default_options, CompilationOptions
@@ -335,20 +360,23 @@ def simple_cythonize(src, destdir=None, cwd=None, logger=None,
     else:
         ori_dir = '.'
     os.chdir(cwd)
-
-    cy_options = CompilationOptions(default_options)
-    cy_options.__dict__.update(cy_kwargs)
-    if logger:
-        logger.info("Cythonizing {0} to {1}".format(
-            src, dstfile))
-    cy_compile([src], cy_options, full_module_name=full_module_name)
-    if os.path.abspath(os.path.dirname(
-            src)) != os.path.abspath(destdir):
-        if os.path.exists(dstfile):
-            os.unlink(dstfile)
-        shutil.move(os.path.join(os.path.dirname(src), c_name),
-                    destdir)
-    os.chdir(ori_dir)
+    try:
+        cy_options = CompilationOptions(default_options)
+        cy_options.__dict__.update(cy_kwargs)
+        if logger:
+            logger.info("Cythonizing {0} to {1}".format(
+                src, dstfile))
+        cy_result = cy_compile([src], cy_options, full_module_name=full_module_name)
+        if cy_result.num_errors > 0:
+            raise ValueError("Cython compilation failed.")
+        if os.path.abspath(os.path.dirname(
+                src)) != os.path.abspath(destdir):
+            if os.path.exists(dstfile):
+                os.unlink(dstfile)
+            shutil.move(os.path.join(os.path.dirname(src), c_name),
+                        destdir)
+    finally:
+        os.chdir(ori_dir)
     return dstfile
 
 
@@ -375,7 +403,7 @@ def src2obj(srcpath, CompilerRunner_=None, objpath=None,
     are dispatched to pyx2obj.
 
     Parameters
-    ==========
+    ----------
     srcpath: path string
         path to source file
     CompilerRunner_: pycompilation.CompilerRunner subclass (optional)
@@ -407,8 +435,10 @@ def src2obj(srcpath, CompilerRunner_=None, objpath=None,
 
     include_dirs = kwargs.pop('include_dirs', [])
     if inc_py:
-        from distutils.sysconfig import get_python_inc, get_config_vars
-        include_dirs += [get_python_inc()]
+        from distutils.sysconfig import get_python_inc
+        py_inc_dir = get_python_inc()
+        if py_inc_dir not in include_dirs:
+            include_dirs.append(py_inc_dir)
 
     if ext.lower() == '.pyx':
         return pyx2obj(srcpath, objpath=objpath,
@@ -422,6 +452,8 @@ def src2obj(srcpath, CompilerRunner_=None, objpath=None,
 
     # src2obj implies not running the linker...
     run_linker = kwargs.pop('run_linker', False)
+    if run_linker:
+        raise CompilationError("src2obj called with run_linker=True")
 
     if only_update:
         if not missing_or_other_newer(objpath, srcpath, cwd=cwd):
@@ -451,7 +483,7 @@ def pyx2obj(pyxpath, objpath=None, interm_c_dir=None, cwd=None,
     destination
 
     Parameters
-    ==========
+    ----------
     pyxpath: path string
         path to Cython source file
     objpath: path string (optional)
@@ -484,7 +516,7 @@ def pyx2obj(pyxpath, objpath=None, interm_c_dir=None, cwd=None,
         keyword arguments passed onto src2obj
 
     Returns
-    =======
+    -------
     Absolute path of generated object file.
 
     """
@@ -494,17 +526,16 @@ def pyx2obj(pyxpath, objpath=None, interm_c_dir=None, cwd=None,
     interm_c_dir = interm_c_dir or os.path.dirname(objpath)
 
     abs_objpath = get_abspath(objpath, cwd=cwd)
-    abs_pyxpath = get_abspath(pyxpath, cwd=cwd)
 
     if os.path.isdir(abs_objpath):
-        pyx_fname = os.path.basename(abs_pyxpath)
+        pyx_fname = os.path.basename(pyxpath)
         name, ext = os.path.splitext(pyx_fname)
         objpath = os.path.join(objpath, name+objext)
 
     cy_kwargs = cy_kwargs or {}
     cy_kwargs['output_dir'] = cwd
     if cplus is None:
-        cplus = pyx_is_cplus(abs_pyxpath)
+        cplus = pyx_is_cplus(pyxpath)
     cy_kwargs['cplus'] = cplus
     if gdb:
         cy_kwargs['gdb_debug'] = True
@@ -512,7 +543,7 @@ def pyx2obj(pyxpath, objpath=None, interm_c_dir=None, cwd=None,
         cy_kwargs['include_path'] = include_dirs
 
     interm_c_file = simple_cythonize(
-        abs_pyxpath, destdir=interm_c_dir,
+        pyxpath, destdir=interm_c_dir,
         cwd=cwd, logger=logger,
         full_module_name=full_module_name,
         only_update=only_update, **cy_kwargs)
@@ -525,16 +556,26 @@ def pyx2obj(pyxpath, objpath=None, interm_c_dir=None, cwd=None,
             include_dirs.append(numpy_inc_dir)
 
     flags = kwargs.pop('flags', [])
-    flags.extend(['-fno-strict-aliasing'])
+    needed_flags = ('-fwrapv', '-pthread')
+    if not cplus:
+        needed_flags += ('-Wstrict-prototypes',)  # not really needed..
+    for flag in needed_flags:
+        if flag not in flags:
+            flags.append(flag)
+
     options = kwargs.pop('options', [])
 
+    if kwargs.pop('strict_aliasing', False):
+        raise CompilationError("Cython req. strict aliasing to be disabled.")
+
     if 'pic' not in options:
         options.append('pic')
     if 'warn' not in options:
         options.append('warn')
 
+    # Let's be explicit about standard
     if cplus:
-        std = kwargs.pop('std', 'c++0x')
+        std = kwargs.pop('std', 'c++98')
     else:
         std = kwargs.pop('std', 'c99')
 
@@ -550,6 +591,7 @@ def pyx2obj(pyxpath, objpath=None, interm_c_dir=None, cwd=None,
         options=options,
         logger=logger,
         inc_py=True,
+        strict_aliasing=False,
         **kwargs)
 
 
@@ -580,7 +622,7 @@ def compile_link_import_py_ext(
     are not recompiled but instead it is imported.
 
     Parameters
-    ==========
+    ----------
     srcs: string
         list of paths to sources
     extname: string
@@ -597,11 +639,11 @@ def compile_link_import_py_ext(
         and link_kwargs useful for convenience e.g. when passing logger
 
     Returns
-    =======
+    -------
     the imported module
 
     Examples
-    ========
+    --------
     >>> mod = compile_link_import_py_ext(['fft.f90', 'convolution.cpp',\
         'fft_wrapper.pyx'], only_update=True)  # doctest: +SKIP
     >>> Aprim = mod.fft(A)  # doctest: +SKIP
@@ -636,10 +678,10 @@ def compile_link_import_strings(codes, build_dir=None, **kwargs):
     provided source code.
 
     Parameters
-    ==========
+    ----------
     codes: iterable of name/source pair tuples
-    build_dir: string
-        path to cache_dir
+    build_dir: string (default: None)
+        path to cache_dir. None implies use a temporary directory.
     **kwargs:
         keyword arguments passed onto `compile_link_import_py_ext`
     """
