@@ -1,8 +1,12 @@
+# -*- coding: utf-8 -*-
+from __future__ import print_function, division, absolute_import
+
 from collections import OrderedDict
 import os
 import re
 import subprocess
-
+import sys
+import warnings
 
 from .util import (
     HasMetaData, get_abspath, FileNotFoundError,
@@ -122,23 +126,38 @@ class CompilerRunner(object):
     def __init__(self, sources, out, flags=None, run_linker=True,
                  compiler=None, cwd=None, include_dirs=None, libraries=None,
                  library_dirs=None, std=None, options=None, define=None,
-                 undef=None, logger=None, preferred_vendor=None,
-                 metadir=None, lib_options=None, only_update=False, **kwargs):
+                 undef=None, strict_aliasing=None, logger=None,
+                 preferred_vendor=None, metadir=None, lib_options=None,
+                 only_update=False, **kwargs):
 
         cwd = cwd or '.'
         metadir = get_abspath(metadir or '.', cwd=cwd)
 
         if hasattr(sources, '__iter__'):
-            self.sources = sources
+            self.sources = list(sources)
         else:
             self.sources = [sources]
 
         self.out = out
         self.flags = flags or []
+        if os.environ.get(self.environ_key_flags):
+            self.flags += os.environ[self.environ_key_flags].split()
         self.metadir = metadir
         self.cwd = cwd
-        if compiler:
-            self.compiler_name, self.compiler_binary = compiler
+        if compiler or os.environ.get(self.environ_key_compiler):
+            if compiler:
+                self.compiler_name, self.compiler_binary = compiler
+            else:
+                self.compiler_binary = os.environ[self.environ_key_compiler]
+                for vk, cn in self.compiler_dict.items():
+                    if cn in self.compiler_binary:
+                        self.compiler_vendor = vk
+                        self.compiler_name = cn
+                        break
+                else:
+                    self.compiler_vendor, self.compiler_name = list(self.compiler_dict.items())[0]
+                    warnings.warn("unsure of what kind of compiler %s is, assuming %s" %
+                                  (self.compiler_binary, self.compiler_name))
             self.save_to_metadata_file(
                 self.metadir, 'vendor',
                 self.compiler_name_vendor_mapping[
@@ -198,6 +217,30 @@ class CompilerRunner(object):
             self.libraries.extend(
                 self.lib_dict[self.compiler_name][lib_opt])
 
+        if strict_aliasing is not None:
+            nsa_re = re.compile("no-strict-aliasing$")
+            sa_re = re.compile("strict-aliasing$")
+            if strict_aliasing is True:
+                if any(map(nsa_re.match, flags)):
+                    raise CompilationError("Strict aliasing cannot be" +
+                                           " both enforced and disabled")
+                elif any(map(sa_re.match, flags)):
+                    pass  # already enforced
+                else:
+                    flags.append('-fstrict-aliasing')
+            elif strict_aliasing is False:
+                if any(map(nsa_re.match, flags)):
+                    pass  # already disabled
+                else:
+                    if any(map(sa_re.match, flags)):
+                        raise CompilationError("Strict aliasing cannot be" +
+                                               " both enforced and disabled")
+                    else:
+                        flags.append('-fno-strict-aliasing')
+            else:
+                raise ValueError("Unknown strict_aliasing={}".format(
+                    strict_aliasing))
+
     @classmethod
     def find_compiler(cls, preferred_vendor, metadir, cwd,
                       use_meta=True):
@@ -240,7 +283,7 @@ class CompilerRunner(object):
                 metadir, 'vendor',
                 cls.compiler_name_vendor_mapping[name])
             if cls.logger:
-                logger.info(
+                cls.logger.info(
                     'Wrote choice of compiler to: metadir')
         return name, path, cls.compiler_name_vendor_mapping[name]
 
@@ -259,7 +302,7 @@ class CompilerRunner(object):
         )
         if self.run_linker:
             cmd += (['-L'+x for x in self.library_dirs] +
-                    ['-l'+x for x in self.libraries] +
+                    [(x if os.path.exists(x) else '-l'+x) for x in self.libraries] +
                     self.linkline)
         counted = []
         for envvar in re.findall('\$\{(\w+)\}', ' '.join(cmd)):
@@ -311,28 +354,34 @@ class CompilerRunner(object):
                              stderr=subprocess.STDOUT,
                              env=env)
         comm = p.communicate()
-        try:
-            self.cmd_outerr = comm[0].decode('utf-8')
-        except UnicodeDecodeError:
-            self.cmd_outerr = comm[0].decode('iso-8859-1')  # win32
+        if sys.version_info[0] == 2:
+            self.cmd_outerr = comm[0]
+        else:
+            try:
+                self.cmd_outerr = comm[0].decode('utf-8')
+            except UnicodeDecodeError:
+                self.cmd_outerr = comm[0].decode('iso-8859-1')  # win32
         self.cmd_returncode = p.returncode
 
         # Error handling
         if self.cmd_returncode != 0:
-            msg = u"Error executing '{0}' in {1}. Command exited with status {2}" + \
-                  u" after givning the following output: {3}\n"
+            msg = "Error executing '{0}' in {1}. Command exited with" + \
+                  " status {2} after givning the following output: {3}\n"
             raise CompilationError(msg.format(
-                u' '.join(self.cmd()), self.cwd, str(self.cmd_returncode),
+                ' '.join(self.cmd()), self.cwd, str(self.cmd_returncode),
                 self.cmd_outerr))
 
         if self.logger and len(self.cmd_outerr) > 0:
-            self.logger.info(u'...with output:\n'+self.cmd_outerr)
+            self.logger.info('...with output:\n'+self.cmd_outerr)
 
         return self.cmd_outerr, self.cmd_returncode
 
 
 class CCompilerRunner(CompilerRunner, HasMetaData):
 
+    environ_key_compiler = 'CC'
+    environ_key_flags = 'CFLAGS'
+
     compiler_dict = OrderedDict([
         ('gnu', 'gcc'),
         ('intel', 'icc'),
@@ -354,18 +403,21 @@ class CCompilerRunner(CompilerRunner, HasMetaData):
             # -march=native not portable and problematic for Mac OSX:
             'fast': ('-O2', '-ffast-math', '-funroll-loops'),
             'openmp': ('-fopenmp',),
+            'debug': ('-g',),
         },
         'icc': {
             'pic': ('-fPIC',),
             'fast': ('-fast',),
             'openmp': ('-openmp',),
             'warn': ('-Wall',),
+            'debug': ('-g',),
         },
         'clang': {
             'pic': ('-fPIC',),
             'warn': ('-Wall', '-Wextra'),
             'fast': ('-O2', '-ffast-math', '-funroll-loops'),
             'openmp': ('-fopenmp',),
+            'debug': ('-g',),
         },
     }
 
@@ -392,6 +444,9 @@ def _mk_flag_filter(cmplr_name):  # helper for class initialization
 
 class CppCompilerRunner(CompilerRunner, HasMetaData):
 
+    environ_key_compiler = 'CXX'
+    environ_key_flags = 'CXXFLAGS'
+
     compiler_dict = OrderedDict([
         ('gnu', 'g++'),
         ('intel', 'icpc'),
@@ -418,7 +473,7 @@ class CppCompilerRunner(CompilerRunner, HasMetaData):
 
     lib_dict = {
         'g++': {
-            'fortran': ('gfortranbegin', 'gfortran'),
+            'fortran': ('gfortran',),
             'openmp': ('gomp',),
         },
         'icpc': {
@@ -452,6 +507,9 @@ class CppCompilerRunner(CompilerRunner, HasMetaData):
 
 class FortranCompilerRunner(CompilerRunner, HasMetaData):
 
+    environ_key_compiler = 'FC'
+    environ_key_flags = 'FFLAGS'
+
     standards = (None, 'f95', 'f2003', 'f2008')  # First is default (F77)
 
     std_formater = {
@@ -468,9 +526,11 @@ class FortranCompilerRunner(CompilerRunner, HasMetaData):
     option_flag_dict = {
         'gfortran': {
             'warn': ('-Wall', '-Wextra', '-Wimplicit-interface'),
+            'debug': ('-g',),
         },
         'ifort': {
             'warn': ('-warn', 'all',),
+            'debug': ('-g',),
         },
     }
 
